C Test program for stren
       PROGRAM STRENEX
       IMPLICIT NONE
C
       DOUBLE PRECISION COMP(8), ST(12)
       DOUBLE PRECISION AE3, ENTR, NITPPM, RATIO, SIGP, SIGS, SIGY, UTS
       DOUBLE PRECISION VACIC, VOLFRA, VP, VS
C
       WRITE(6,*) 'Carbon wt%'
       READ(5,*) COMP(1)
       WRITE(6,*) 'Silicon wt%'
       READ(5,*) COMP(2)
       WRITE(6,*) 'Manganese wt%'
       READ(5,*) COMP(3)
       WRITE(6,*) 'Nickel wt%'
       READ(5,*) COMP(4)
       WRITE(6,*) 'Molybdenum wt%'
       READ(5,*) COMP(5)
       WRITE(6,*) 'Chromium wt%'
       READ(5,*) COMP(6)
       WRITE(6,*) 'Vanadium wt%'
       READ(5,*) COMP(7)
C
       WRITE(6,*) 'Nitrogen Concentration (ppm by wt)'
       READ(5,*) NITPPM
C
       WRITE(6,*) 'Volume fraction of allotriomorphic ferrite'
       READ(5,*) VOLFRA
C
       WRITE(6,*) 'Volume fraction of acicular ferrite'
       READ(5,*) VACIC
C
       CALL MAP_STEEL_STREN(COMP, NITPPM, VOLFRA, VACIC,
     &  VP, VS, SIGP, SIGS, SIGY, UTS, AE3, RATIO, ENTR, ST)
C
       WRITE(6,1000) 
       WRITE(6,1005) VP, VS
       WRITE(6,1010) SIGP, SIGS
       WRITE(6,*)
       WRITE(6,1015) SIGY
       WRITE(6,1020) UTS
       WRITE(6,1030) RATIO
       WRITE(6,*) 
       WRITE(6,1025) AE3
       WRITE(6,*) 
       WRITE(6,1035) ENTR
       WRITE(6,*) 
       WRITE(6,1040)
       WRITE(6,1045) ST(1)
       WRITE(6,1050) ST(2)
       WRITE(6,1055) ST(3)
       WRITE(6,1060) ST(12)
       WRITE(6,1065) ST(4)
       WRITE(6,1070) ST(5)
       WRITE(6,1075) ST(6)
       WRITE(6,1080) ST(7)
       WRITE(6,1085) ST(8)
       WRITE(6,1090) ST(9)
       WRITE(6,1095) ST(10)
       WRITE(6,1100) ST(11)
C
       STOP
1000   FORMAT(16X, 'Primary', 9X, 'Secondary')
1005   FORMAT('Microstructure', D13.5, 3X,  D13.5)
1010   FORMAT('Strength (Mpa)', D13.5, 3X,  D13.5)
1015   FORMAT('Overall yield strength (MPa)', 7X, D13.5)
1020   FORMAT('Overall tensile strength(MPa) ', 5X D13.5)
1025   FORMAT('Paraequilibrium temperature ', D13.5)
1030   FORMAT('Ratio of yield to tensile strength ', D13.5)
1035   FORMAT('Toughness scatter ', D13.5)
1040   FORMAT('Strength components due to :-')
1045   FORMAT('Allo. ferrite', 7X, D13.5)
1050   FORMAT('Acicular ferrite    ', D13.5)
1055   FORMAT('Widman. ferrite', 5X, D13.5)
1060   FORMAT('Secondary hardening', 1X, D13.5)
1065   FORMAT('Nitrogen', 12X, D13.5)
1070   FORMAT('Manganese', 11X, D13.5)
1075   FORMAT('Silicon', 13X, D13.5)
1080   FORMAT('Nickel', 14X, D13.5)
1085   FORMAT('Molybdenum', 10X, D13.5)
1090   FORMAT('Chromium', 12X, D13.5)
1095   FORMAT('Vanadium',12X, D13.5)
1100   FORMAT('Iron',16X, D13.5)
       END


C Subroutine MAP_STEEL_STREN
C
C Provided by :-
C
C  H.K.D.H. Bhadeshia,
C  Phase Transformations Group,
C  Department of Materials Science and Metallurgy,
C  University of Cambridge,
C  Cambridge, U.K.
C
C Purpose :-
C
C Given the composition, volume fraction of allotriomorphic ferrite and volume 
C fraction of acicular ferrite of a steel weld, this routine calculates the 
C overall yield strength, the overall tensile strength and the paraequilibrium 
C temperature of the material [1,2]. It also calculates the secondary hardening 
C effect of molybdenum and gives as output parameters the strength components  
C due to the microstructure and each element present.
C The secondary hardening effect of molybdenum is accounted for using Evans' 
C MMA molybdenum data [3], and the assumption that molybdenum does not
C influence the fractions of primary and secondary regions in any exceptional
C way. There is an arbitrary limit before the carbon-molybdenum effect
C manifests itself.</p>
C
C
C References :-
C
C 1. Computer Technology in Welding, pt.II.
C
C 2. A. A. B. Sugden and H. K. D. H. Bhadeshia, "A Model for the Prediction of 
C    Strength in Steel Welds", Metallurgical Transactions A, Vol. 19, 1988,
C    pp. 1597-1602
C
C 3. H.K.D.H. Bhadeshia, Mathematical Modelling of Weld Phenomena I, 
C    eds H. Cerjak and K.E. Easterling, The Institute of Materials, London, 
C    1993, pp 109-182.
C
C Input parameters :-
C
C COMP contains the concentrations by percentage weight of carbon, silicon,
C  manganese, nickel, molydenum, chromium, vanadium, and iron.
C NITPPM is the nitrogen concentration in parts per million by weight. If it
C  is zero on input, then it is calculated using the method in [1].
C VOLFRA is the volume fraction of allotriomorphic ferrite.
C VACIC is the volume fraction of acicular ferrite.
C
C Output parameters :-
C
C VP is the primary microstructure at the interpass temperature.
C VS is the secondary microstructure at the interpass temperature.
C SIGP is the strength of the primary microstructure (in MPa).
C SIGS is the strength of the secondary microstructure (in MPa).
C SIGY is the overall yield strength (in MPa).
C UTS is the overall tensile strength (in MPa).
C AE3 is the paraequilibrium temperature.
C RATIO is the ratio of yield to tensile strength.
C ENTR is the scatter in toughness (0 is low, 1 is high).
C ST contains the components of strength due to:-
C   ST(1)  - allotriomorphic ferrite.
C   ST(2)  - acicular ferrite.
C   ST(3)  - Widmanstatten ferrite.
C   ST(4)  - nitrogen.
C   ST(5)  - manganese.
C   ST(6)  - silicon.
C   ST(7)  - nickel.
C   ST(8)  - molybdenum.
C   ST(9)  - chronium.
C   ST(10) - vanadium.
C   ST(11) - iron.
C   ST(12) - secondary hardening effect of Mo.
C
C Auxiliary subroutine :-
C
C   MAP_STEEL_AA3
C
C Copyright Dr. H. K. D. H. Bhadeshia, University of Cambridge
C December 1987
C Version 1.4
C
C NITPPM = Nitrogen concentration in ppmw. If zero, then calculated,
C according to Computer Technology in Welding, pt.II
C
       SUBROUTINE MAP_STEEL_STREN(COMP, NITPPM, VOLFRA, VACIC,
     &  VP, VS, SIGP, SIGS, SIGY, UTS, AE3, RATIO, ENTR, ST)
C
       IMPLICIT NONE
C
C Input/output arguments :-
C
       DOUBLE PRECISION COMP(8), ST(12)
       DOUBLE PRECISION AE3, ENTR, NITPPM, RATIO, SIGP, SIGS, SIGY, UTS
       DOUBLE PRECISION VACIC, VOLFRA, VP, VS, R
C
C Local variables :-
C
       DOUBLE PRECISION VOLW, MOSEC, CSEC, CDUM, RATIO, ENTR
       INTEGER I1, I2, I3
C
       PARAMETER(R=8.31451D+00)
       I1=873
       I2=1173
       I3=30
C
C Begin subroutine proper :-
C
       CALL MAP_STEEL_AA3(COMP,AE3,R,I1,I2,I3)
C
       VP = 3.313 - 0.0037*AE3
       IF(VP .GT. 1.0) THEN
         GOTO 10
       END  IF
       GOTO 20
C
10     CONTINUE
       VP   = 1.0
C
20     CONTINUE
       VS   = 1.0-VP
       VOLW = 1.0-VOLFRA-VACIC
C
       ST(1) = VOLFRA*27.0
       ST(2) = VACIC*402.0
       ST(3) = VOLW*486.0
       ST(4) = NITPPM*42.8/100.0
       ST(5) = COMP(3)*40.8
       ST(6) = COMP(2)*94.5
       ST(7) = COMP(4)*33.2
C
C 2 October 1993; modification to account for the secondary hardening effect
C   of Mo, using Evans's MMA Mo data and the assumption that Mo does not
C   influence the fractions of primary and secondary regions in any 
C   exceptional way.  There is an arbitrary limit before the C-Mo effect
C   manifests, and there is a maximum limit to the C-Mo effect.  See H.K.D.H. 
C   Bhadeshia, <cite>Mathematical Modelling of Weld Phenomena I</cite>, eds 
C   H. Cerjak and K.E. Easterling, The Institute of Materials, London, 1993,
C   pp 109-182.C
C
C MOSEC and CSEC are the additional contributions due to secondary hardening
C
       MOSEC = 132.00*COMP(5)
       IF(COMP(1) .GE. 0.037 .AND. COMP(5) .GE. 0.10D+00) THEN
          CDUM = COMP(1)
          IF(COMP(1) .GE. 0.05) THEN
             CDUM = 0.05
          END  IF
          CSEC = (CDUM - 0.037)*4700.0
       ELSE
          CSEC = 0.0D+00
       ENDIF
C
       ST(8)  = COMP(5)*16.0 + MOSEC + CSEC
       ST(9)  = COMP(6)*7.5
       ST(10) = COMP(7)*4.5
       ST(11) = 219.0
       ST(12) = 68.0
C
       SIGS = ST(4)+ST(5)+ST(6)+ST(7)+ST(8)+ST(9)+
     &        ST(10)+ST(11)+ST(12)
       SIGP = ST(1)+ST(2)+ST(3)+ST(4)+ST(5)+ST(6)+
     &        ST(7)+ST(8)+ST(9)+ST(10)+ST(11)
       SIGY = VS*SIGS+VP*SIGP
C
       ST(1)  = ST(1) *VP
       ST(2)  = ST(2) *VP
       ST(3)  = ST(3) *VP
       ST(12) = ST(12)*VS
C
       UTS   = 112.4+0.9153*SIGY
       RATIO = SIGY/UTS
C
       IF(VACIC .EQ. 1.0D+00 .OR. VOLFRA .EQ. 1.0D+00 .OR .
     & VOLW .EQ. 1.00) THEN
          GOTO 30
       END  IF
C
       ENTR = (VOLFRA*LOG(VOLFRA+1.0D-06) +
     & VACIC*LOG(VACIC+1.0D-06)
     & + VOLW*LOG(VOLW+1.0D-06))/(LOG(1.0D+00/3.0D+00))
       GOTO 40
C
30     CONTINUE
       ENTR=0.0D+00
C
40     CONTINUE
       RETURN
       END
C
C
C
      SUBROUTINE MAP_STEEL_AA3(AC,AE3T,R,I1,I2,I3)
C
      IMPLICIT NONE
      DOUBLE PRECISION DTQ(40),DXEQ(40),XEQ,C(8),AC(8)
      DOUBLE PRECISION R,W,XBAR,T10,T20,T,F,MAP_STEEL_ENERGY
      DOUBLE PRECISION TEQ,MAP_STEEL_AFEG,MAP_STEEL_DAFEG,ETEQ
      DOUBLE PRECISION CTEMP,AINCET,SLOPE,CORR,AJ,AE3T
      INTEGER I1,I2,I3,J8,I,J,IT
C
C This ensures that AC remains unchanged after being called by OMEGA
         DO 10 J=1,8
         C(J)=AC(J)
 10      CONTINUE
C
      I=0
      CALL MAP_STEEL_OMEGA(C,W,XBAR,T10,T20,0)
C
C It is assumed that the Ae3' temperature lies between I1 and I2, in Kelvin
C I3 defines the temperature interval over which the calculations are conducted
C
C Take a guess for XEQ
 20   XEQ=0.1
      J8=0
C
C Calculate a set of XEQ values as a function of T
C For method, see equation 2, Acta Metallurgica, 28 (1980) 1265-1273.
C   The equation is solved by iteration. Hence, AFEG is the natural logarithm
C   of the activity of iron in austenite, DAFEG is the differential of that
C   with respect to carbon (XEQ). The iteration is based on the Newton-Raphson
C   method. The iteration stops when XEQ becomes very small.
C   The precise Ae3' temperature is found by linear regression of the
C   temperature XEQ pairs by the ANAL.
C
        DO 30 IT=I1,I2,I3
          J8=J8+1
          T=IT
          F=MAP_STEEL_ENERGY(T,T10,T20)
          AJ=1.0-DEXP(-W/(R*T))
 50         TEQ=MAP_STEEL_AFEG(XEQ,AJ)*R*T-F
            IF (DABS(TEQ) .LT. 1.0) GOTO 60
            ETEQ=MAP_STEEL_DAFEG(XEQ,AJ)*R*T
            XEQ=XEQ-TEQ/ETEQ
            GOTO 50
C Calculations stop when XEQ becomes very small
 60       IF (XEQ .LT. 0.001) GOTO 70
          CTEMP=IT-273.0
          DTQ(J8)=CTEMP
          DXEQ(J8)=XEQ
 30     CONTINUE
 70     J8=J8-1
C
C Linear regression to find exact Ae3' temperature for alloy of interest
        CALL MAP_UTIL_ANALY(J8,0,AINCET,SLOPE,CORR,DTQ,DXEQ)
        AE3T=(C(1)-AINCET)/SLOPE
C
C Repeat analysis to a finer precision for Ae3'
        IF(I .GT. 0) GOTO 100
        I=AE3T
        I1=I-9+273
        I2=I+9+273
        I3=3
      GOTO 20
100   RETURN
      END
C
C
C
      DOUBLE PRECISION FUNCTION MAP_STEEL_ENERGY(T,T10,T20)
      IMPLICIT NONE
      DOUBLE PRECISION T,T10,T20,F,T7,T8
      T7=T-100.0D+0*T20
      T8=T7-1140.0D+0
      IF (T7 .LT. 300) F=1.38*T7-1499.0
      IF (T7.LT.700.AND.T7.GE.300)  F=1.65786*T7-1581.0
      IF (T7.LT.940.AND.T7.GE.700)  F=1.30089*T7-1331.0
      IF (T7.LT.940) GOTO 10
      F=T8*(-3.58434D-9*T8 + 2.70013D-6) - 1.04923D-3
      F=T8*(F*T8 + 0.26557D+0) - 8.88909D+0
 10   MAP_STEEL_ENERGY=(141.0*T10 + F)*4.187
      RETURN
      END
C
C
C
      SUBROUTINE MAP_UTIL_ANALY(J8,J9,CONST,SLOPE,CORR,X,Y)
      IMPLICIT NONE
      DOUBLE PRECISION X(100),Y(100),AX,AY,AX2,AY2,AXY
      DOUBLE PRECISION CONST,SLOPE,CORR
      INTEGER I1,J8,J9,I
      I1=J8
      J8=J8-J9
      AX=0.0D+00
      AY=0.0D+00
      AX2=0.0D+00
      AY2=0.0D+00
      AXY=0.0D+00
      DO 1 I=1,J8
      AX=AX+X(I)
      AY=AY+Y(I)
      AXY=AXY+X(I)*Y(I)
      AX2=AX2+X(I)*X(I)
      AY2=AY2+Y(I)*Y(I)
1     CONTINUE
      CONST=(AY*AX2-AX*AXY)/(J8*AX2-AX*AX)
      SLOPE=((J8*AXY-AX*AY)/(J8*AX2-AX*AX))
      CORR=(J8*AXY-AX*AY)/(DSQRT((J8*AX2-AX*AX)*
     &(J8*AY2-AY*AY)))
      J8=I1
      RETURN
      END
C
C
C
      DOUBLE PRECISION FUNCTION MAP_STEEL_DAFEG(XEQ,AJ)
      IMPLICIT NONE
      DOUBLE PRECISION ETEQ,ETEQ2,DEQ,XEQ,AJ,MAP_STEEL_DAFEG
      DEQ=DSQRT(1-2*(1+2*AJ)*XEQ+(1+8*AJ)*XEQ*XEQ)
      ETEQ=5*((1/(XEQ-1))+2/(1-2*XEQ))
      ETEQ2=6*((4*AJ-1-(0.5/DEQ)*(-2-4*AJ+2*XEQ+16*XEQ*AJ))
     &/(1-2*AJ+(4*AJ-1)*XEQ-DEQ))+6*(4*AJ/(2*AJ*(2*XEQ-1)))
      MAP_STEEL_DAFEG=ETEQ+ETEQ2
      RETURN
      END
C
C
      SUBROUTINE MAP_STEEL_OMEGA(C,W,XBAR,T10,T20,J)
C SUBROUTINE TO CALCULATE THE CARBON CARBON INTERACTION ENERGY IN
C AUSTENITE, AS A FUNCTION OF ALLOY COMPOSITION.  BASED ON .MUCG18
C THE ANSWER IS IN JOULES PER MOL.   **7 OCTOBER 1981**
      IMPLICIT NONE
      DOUBLE PRECISION C(8),W,P(8),B1,B2,Y(8),T10,T20,B3,XBAR
      INTEGER IU,J
      B3=0.0D+00
      IF(J .EQ. 1) GOTO 2
      C(8)=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)
      C(8)=100.0D+00-C(8)
      C(8)=C(8)/55.84D+00
      C(1)=C(1)/12.0115D+00
      C(2)=C(2)/28.09D+00
      C(3)=C(3)/54.94D+00
      C(4)=C(4)/58.71D+00
      C(5)=C(5)/95.94D+00
      C(6)=C(6)/52.0D+00
      C(7)=C(7)/50.94D+00
2     B1=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)
      DO 107 IU=2,7
      Y(IU)=C(IU)/C(8)
107   CONTINUE
      DO 106 IU=1,8
      C(IU)=C(IU)/B1
106   CONTINUE
      XBAR=C(1)
      XBAR=DINT(10000.0D+00*XBAR)
      XBAR=XBAR/10000
      B2=0.0D+00
      T10=Y(2)*(-3)+Y(3)*2+Y(4)*12+Y(5)*(-9)+Y(6)*(-1)+Y(7)*(-12)
      T20=-3*Y(2)-37.5*Y(3)-6*Y(4)-26*Y(5)-19*Y(6)-44*Y(7)
C
C     Polynomials representing carbon-carbon interaction energy (J/mol)
C     in austenite as a function of the molecular fraction of individual solutes.
C
      P(2)=-2.4233D+07+6.9547D+07*C(2)
      P(2)=+3.864D+06+P(2)*C(2)
      P(2)=+45802.87+(-280061.63+P(2)*C(2))*C(2) 
      P(2)=2013.0341+(763.8167+P(2)*C(2))*C(2)
      P(3)=2.0119D+06+(3.1716D+07-1.3885D+08*C(3))*C(3)
      P(3)=+6287.52+(-21647.96+P(3)*C(3))*C(3)
      P(3)=2012.067+(-1764.095+P(3)*C(3))*C(3)
      P(4)=-2.4968D+07+(1.8838D+08-5.5531D+08*C(4))*C(4)
      P(4)=-54915.32+(1.6216D+06+P(4)*C(4))*C(4)
      P(4)=2006.8017+(2330.2424+P(4)*C(4))*C(4)
      P(5)=-1.3306D+07+(8.411D+07-2.0826D+08*C(5))*C(5)
      P(5)=-37906.61+(1.0328D+06+P(5)*C(5))*C(5)
      P(5)=2006.834+(-2997.314+P(5)*C(5))*C(5)
      P(6)=+8.5676D+06+(-6.7482D+07+2.0837D+08*C(6))*C(6)
      P(6)=+33657.8+(-566827.83+P(6)*C(6))*C(6)
      P(6)=2012.367+(-9224.2655+P(6)*C(6))*C(6)
      P(7)=+5411.7566
     &+(250118.1085-4.1676D+06*C(7))*C(7)
      P(7)=2011.9996+(-6247.9118+P(7)*C(7))*C(7)
      DO 108 IU=2,7
      B3=B3+P(IU)*Y(IU)
      B2=B2+Y(IU)
108   CONTINUE
      IF (B2 .EQ. 0.0D+00) GOTO 455
      W=(B3/B2)*4.187
      GOTO 456
455   W=8054.0
456   CONTINUE
      RETURN
      END
C
C
C
C
C H. K. D. H. Bhadeshia, 1984
C Natural logarithm of the activity of iron in austenite
C
      DOUBLE PRECISION FUNCTION MAP_STEEL_AFEG(XEQ,AJ)
      IMPLICIT NONE
      DOUBLE PRECISION XEQ,AJ,DEQ,TEQ
C
      DEQ=DSQRT(1-2*(1+2*AJ)*XEQ+(1+8*AJ)*XEQ*XEQ)
      TEQ=5.0*DLOG((1-XEQ)/(1-2*XEQ))
      TEQ=TEQ+6.0*DLOG((1-2*AJ+(4*AJ-1)*XEQ-DEQ)/(2*AJ*(2*XEQ-1)))
      MAP_STEEL_AFEG=TEQ
C
      RETURN
      END
C
